/*******************************************************************
 * Fritz Fun                                                       *
 * Created by Jan-Michael Brummer                                  *
 * All parts are distributed under the terms of GPLv2. See COPYING *
 *******************************************************************/

/**
 * \file notification.c
 * \brief Notification handling
 */

#include <ffgtk.h>
#include <connection.h>
#include <notification.h>

static struct sNotification *psNotification = NULL;

#ifdef HAVE_FAXOPHONE
/**
 * \brief Accept incoming call
 * \param psWidget button widget
 * \param pData popup widget pointer
 */
void acceptClicked( GtkWidget *psWidget, gpointer pData ) {
	struct sConnection *psConnection = pData;
	gchar *pnNumber = psConnection -> pnNumber;

	/* Set dialog information */
	dialSetNumber( pnNumber );
	dialingDialog( pnNumber, _( "Softphone" ) );

	closeNotification( psConnection );

	/* Stop ring tone */
	StopRingtone();

	/* Ignore next monitor messages */
	connectionIgnore( psConnection );

	/* Pickup call */
	softphonePickup();
}

/**
 * \brief Deny incoming call
 * \param psWidget button widget
 * \param pData popup widget pointer
 */
void denyClicked( GtkWidget *psWidget, gpointer pData ) {
	struct sConnection *psConnection = pData;

	softphoneHangup();

	closeNotification( psConnection );
}
#endif

/**
 * \brief Close notification window
 * \param pData popup window
 * \return FALSE
 */
gboolean closeNotification( gpointer pData ) {
	struct sConnection *psConnection = pData;
	gboolean bRet = FALSE;

	if ( psConnection -> nNotifyTimeout != 0 ) {
		g_source_remove( psConnection -> nNotifyTimeout );
		psConnection -> nNotifyTimeout = 0;
	}

	/* Destroy popup widget */
	if ( psConnection -> pNotifyWindow != NULL ) {
		psNotification -> CloseNotification( psConnection -> pNotifyWindow );
		psConnection -> pNotifyWindow = NULL;
	}

	return bRet;
}
/**
 * \brief Show popup call notification
 * \param nType call type
 * \param psPerson person structure
 * \param pnNumber remote number
 * \param pnLocal local number
 * \return popup window
 */
void *showNotification( struct sConnection *psConnection, eCallType nType, struct sPerson *psPerson, gchar *pnNumber, gchar *pnLocal ) {
	psConnection -> pNotifyWindow = psNotification -> ShowNotification( psConnection, nType, psPerson, pnNumber, pnLocal );

	return psConnection -> pNotifyWindow;
}

/**
 * \brief Update person information in popup window
 * \param psPopup popup window
 * \param nType call type
 * \param pnName remote name
 * \param pnStreet remote street
 * \param pnCity remote city
 */
void updateNotification( void *psPopup, eCallType nType, gchar *pnName, gchar *pnStreet, gchar *pnCity ) {
	psNotification -> UpdateNotification( psPopup, nType, pnName, pnStreet, pnCity );
}

/**
 * \brief Initialize notification system
 * \return 1 on success, otherwise error
 */
int InitNotification( void ) {
#ifdef HAVE_LIBNOTIFY
	psNotification = InitNotificationNotify();
#endif
	if ( psNotification == NULL ) {
		psNotification = InitNotificationGtk();
	}

	return psNotification != NULL;
}
