/*******************************************************************
 * Fritz Fun                                                       *
 * Created by Jan-Michael Brummer                                  *
 * All parts are distributed under the terms of GPLv2. See COPYING *
 *******************************************************************/

/**
 * \file thread.c
 * \brief Thread wrapper functions
 */

#include <ffgtk.h>
#include <thread.h>

/**
 * \brief Initialize thread structure
 * \param psThread thread structure
 * \param pnName thread name
 */
void InitThread( struct sThread *psThread, gchar *pnName ) {
	if ( !g_thread_supported() ) {
		g_thread_init( NULL );
	}

	psThread -> psThread = NULL;
	psThread -> nStopFlag = 0;
	psThread -> pnName = g_strdup( pnName );
}

/**
 * \brief Start thread
 * \param psThread thread structure
 * \param Handler handler function
 * \param pParam optional handler param
 * \return error code
 */
int StartThread( struct sThread *psThread, gpointer ( *Handler )( gpointer ), gpointer pParam ) {
	if ( !g_thread_supported() ) {
		g_thread_init( NULL );
	}

	if ( psThread -> psThread == NULL ) {
		psThread -> nStopFlag = 0;

		psThread -> psThread = CREATE_THREAD( psThread -> pnName, Handler, pParam );
		if ( psThread -> psThread != NULL ) {
			return 0;
		}

		return -1;
	}

	return 1;
}

/**
 * \brief Stop running thread
 * \param psThread thread structure
 */
void StopThread( struct sThread *psThread ) {
	GThread *psToStop = psThread -> psThread;

	if ( psToStop != NULL ) {
		psThread -> nStopFlag = 1;
		psThread -> psThread = NULL;

		g_thread_join( psToStop );
		psThread -> nStopFlag = 0;
	}
}

/**
 * \brief Tead if thread should stop
 * \param psThread thread structure
 * \return stop flag value
 */
int ThreadIsStopping( struct sThread *psThread ) {
	return psThread -> nStopFlag;
}

/**
 * \brief Tead if thread is running
 * \param psThread thread structure
 * \return running flag
 */
int ThreadIsRunning( struct sThread *psThread ) {
	return psThread -> psThread != NULL;
}
