/*******************************************************************
 * Fritz Fun                                                       *
 * Created by Jan-Michael Brummer                                  *
 * All parts are distributed under the terms of GPLv2. See COPYING *
 *******************************************************************/

/**
 * \file trayicon.c
 * \brief Trayicon related functions
 */

#include <ffgtk.h>
#include <menucallbacks.h>
#include <trayicon.h>
#include <osdep/osdep.h>

/**
 * \brief Add diversity information to submenu
 * \param psSubMenu add all information to this submenu
 */
static void addDiversity( GtkWidget *psSubMenu ) {
	struct sDiversity *psDiversity = NULL;
	GList *psList = NULL;
	GtkWidget *psItem = NULL;

	psItem = gtk_image_menu_item_new_with_label( _( "Diversity" ) );
	GtkWidget *psSubMenu3 = gtk_menu_new();
	GtkWidget *psDiversityImage = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psDiversityImage );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psItem ), psSubMenu3 );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );

	psList = routerGetDiversity();
	if ( psList == NULL || g_list_length( psList ) == 0 ) {
		gtk_widget_set_sensitive( psItem, FALSE );
		return;
	}

	for ( psList = routerGetDiversity(); psList != NULL && psList -> data != NULL; psList = psList -> next ) {
		psDiversity = psList -> data;

		psItem = gtk_check_menu_item_new_with_label( psDiversity -> pnName );
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), psDiversity -> bSet );
		gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu3 ), psItem );
		g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconSetDiversity ), psDiversity );
	}
}

void addLastcalls( GtkWidget *psMenuWidget ) {
	GtkWidget *psItem = NULL;

	psItem = gtk_image_menu_item_new_with_label( _( "Last calls" ) );
	GtkWidget *psLastCalls = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psLastCalls );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	
	addLastcallsMenu( psItem );
}

GtkWidget *createMenu( gboolean bAddQuitItem ) {
	GtkWidget *psItem;
	GtkWidget *psMenuWidget;

	psMenuWidget = gtk_menu_new();

	/* Callmonitor */
	psItem = gtk_image_menu_item_new_with_label( _( "Call Monitor" ) );
	GtkWidget *psMonitor = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psMonitor );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconToggled ), NULL );

	/* Addressbook */
	psItem = gtk_image_menu_item_new_with_label( _( "Address Book" ) );
	GtkWidget *psAddress = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psAddress );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconAddressBook ), NULL );

	/* Last calls */
	addLastcalls( psMenuWidget );

	/* Functions */
	GtkWidget *psFunctions = gtk_image_menu_item_new_with_label( _( "Functions" ) );
	GtkWidget *psSubMenu1 = gtk_menu_new();
	GtkWidget *psFunctionsImage = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psFunctions ), psFunctionsImage );

	/* Functions - Call number */
	psItem = gtk_image_menu_item_new_with_label( _( "Call number" ) );
	GtkWidget *psCallMonitor = GTK_WIDGET( gtk_image_new_from_pixbuf( getTypeIcon( 3 ) ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psCallMonitor );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu1 ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), NULL );
	gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );

	/* Functions - Reconnect */
	psItem = gtk_image_menu_item_new_with_label( _( "Reconnect" ) );
	GtkWidget *psReconnect = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psReconnect );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu1 ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconReconnect ), NULL );

	/* Functions - Copy IP */
	psItem = gtk_image_menu_item_new_with_label( _( "Copy IP" ) );
	GtkWidget *psClipboard = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_COPY, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psClipboard );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu1 ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconIpToClipboard ), NULL );

	/* Functions - Diversity */
	addDiversity( psSubMenu1 );

	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psFunctions ), psSubMenu1 );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psFunctions );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );

	/* Preferences */
	psItem = gtk_image_menu_item_new_with_label( _( "Preferences" ) );
	GtkWidget *psPref = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psPref );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( preferences ), NULL );

	/* Profiles */
	GtkWidget *psProfilesMenu = gtk_image_menu_item_new_with_label( _( "Profiles" ) );
	GtkWidget *psSubMenu = gtk_menu_new();
	GtkWidget *psCall = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_INDEX, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psProfilesMenu ), psCall );

	/* Profiles - Manage */
	psItem = gtk_image_menu_item_new_with_label( _( "Manage" ) );
	GtkWidget *psManage = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_INDEX, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psManage );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( ProfilesManage ), NULL );

	/* Profiles - Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );

	/* Profiles - Profile list */
	GList *psList = getProfiles();
	struct sProfile *psProfile = NULL;
	psItem = NULL;
	GSList *psGroup = NULL;

	while ( psList != NULL ) {
		psProfile = psList -> data;
		if ( psProfile != NULL ) {
			psItem = gtk_radio_menu_item_new_with_label( psGroup, psProfile -> pnName );
			psGroup = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM( psItem ) );
			if ( psProfile == getActiveProfile() ) {
				gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), TRUE );
			}
			gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconProfile ), psProfile -> pnName );
		}
		psList = psList -> next;
	}

	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psProfilesMenu ), psSubMenu );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psProfilesMenu );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );

	/* Mute */
	psItem = gtk_check_menu_item_new_with_label( _( "Mute" ) );
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), callMonitorGetMute( getActiveProfile() ) );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconMute ), NULL );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );

	/* Help */
	psItem = gtk_image_menu_item_new_with_label( _( "Help" ) );
	GtkWidget *psHelp = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_HELP, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psHelp );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconHelp ), NULL );

	/* About */
	psItem = gtk_image_menu_item_new_with_label( _( "About" ) );
	GtkWidget *psAbout = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psAbout );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( aboutDialog ), NULL );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );

	if ( bAddQuitItem ) {
		/* Quit */
		psItem = gtk_image_menu_item_new_with_label( _( "Quit" ) );
		GtkWidget *psQuit = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_QUIT, GTK_ICON_SIZE_MENU ) );
		gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psQuit );
		gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
		g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( destroy ), NULL );
	}

	return psMenuWidget;
}
