/*******************************************************************
 * Fritz Fun                                                       *
 * Created by Jan-Michael Brummer                                  *
 * All parts are distributed under the terms of GPLv2. See COPYING *
 *******************************************************************/

/**
 * \file ab_speedfon.c
 * \brief SpeedFon addressbook plugin
 */

#include <ffgtk.h>
#include <sys/stat.h>

extern const gchar *getProfilesDir( void );

#define VALID_NUMBER( x ) ( ( x != NULL ) && ( strlen( x ) > 0 ) )

/** Maximal speed fon chunk size */
static int nSpeedfonMaxSize = 2048;

/** Swap first/last name */
static gboolean speedfonGetSwap( void ) {
	return prefsGetBool( getActiveProfile(), "/plugins/speedfon/swap" );
}

/** Swap first/last name */
static void speedfonSetSwap( gboolean bSwap ) {
	prefsSetBool( getActiveProfile(), "/plugins/speedfon/swap", bSwap );
}
		
/**
 * \brief Read book list
 * \return error code
 */
static int speedfonReadBook( void ) {
	struct sUrlHandler *psHandler;
	gchar anUrl[ 1024 ];
	gint nError = -1;
	struct sProfile *psProfile = getActiveProfile();
	gint nStart = 0;
	gint nEnd = -1;

	if ( routerLogin( psProfile ) == -1 ) {
		return -1;
	}

	snprintf( anUrl, sizeof( anUrl ), "%s%s/cgi-bin/webcm?getpage=../html/hcti_telefon_kuwa.htm", /*getRouterProtocol( psProfile )*/"http://", routerGetHost( psProfile ) );
	psHandler = urlHandler( anUrl, /*getRouterPort( psProfile )*/80);
	nError = readUrl( psHandler, psProfile );

	if ( nError == 0 ) {
		gchar *pnName = NULL;
		gchar *pnNumber = NULL;
		gchar *pnTmp = NULL;
		int nCount = 0;

		saveDebugData( "speedport_kuwa.html", psHandler -> pnData, psHandler -> nSize );

		while ( nStart != -1 ) {
			nStart = findString( psHandler -> pnData, nStart, "Kurzwahl: " );
			Debug( KERN_DEBUG, "nStart: %d\n", nStart );
			if ( nStart == -1 ) {
				break;
			}
			nStart = findString( psHandler -> pnData, nStart, ">" );
			Debug( KERN_DEBUG, "nStart: %d\n", nStart );
			if ( nStart == -1 ) {
				break;
			}
			nEnd = findString( psHandler -> pnData, nStart + 1, "<" );
			if ( nEnd == -1 ) {
				break;
			}

			pnName = getSubString( psHandler -> pnData, nStart + 1, nEnd - nStart - 1 );
			if ( pnName == NULL ) {
				nStart++;
				continue;
			}

			nStart = findString( psHandler -> pnData, nStart, "colLast" );
			Debug( KERN_DEBUG, "nStart: %d\n", nStart );
			if ( nStart == -1 ) {
				break;
			}
			nStart = findString( psHandler -> pnData, nStart, ">" );
			Debug( KERN_DEBUG, "nStart: %d\n", nStart );
			if ( nStart == -1 ) {
				break;
			}
			nEnd = findString( psHandler -> pnData, nStart + 1, "<" );
			if ( nEnd == -1 ) {
				break;
			}

			pnNumber = getSubString( psHandler -> pnData, nStart + 1, nEnd - nStart - 1 );
			if ( pnNumber == NULL ) {
				g_free( pnName );
				nStart++;
				continue;
			}

			if ( pnName != NULL && pnNumber != NULL ) {
				/* Create person hash table, fill structure and add them to internal list */
				GHashTable *psTable = g_hash_table_new( NULL, NULL );
				gchar *pnFirstName;
				gchar *pnLastName;
				gchar *pnId = g_strdup_printf( "%d", nCount );
				gchar *pnRealName = convertEntities( pnName );

				Debug( KERN_DEBUG, "pnName: '%s' = '%s'\n", pnRealName, pnNumber );

				pnTmp = strchr( pnRealName, ' ' );
				if ( pnTmp != NULL ) {
					gint nLen = 0;
					pnLastName = strrchr( pnRealName, ' ' ) + 1;
					nLen = strlen( pnRealName ) - strlen( pnLastName ) - 1;
					pnFirstName = g_malloc0( nLen + 1 );
					strncpy( pnFirstName, pnRealName, nLen );
					pnFirstName[ nLen ] = '\0';
				} else {
					pnFirstName = NULL;
					pnLastName = pnRealName;
				}

				AddInfo( psTable, PERSON_ID, pnId );
				if ( speedfonGetSwap() == FALSE ) {
					AddInfo( psTable, PERSON_FIRST_NAME, pnFirstName );
					AddInfo( psTable, PERSON_LAST_NAME, pnLastName );
				} else {
					AddInfo( psTable, PERSON_FIRST_NAME, pnLastName );
					AddInfo( psTable, PERSON_LAST_NAME, pnFirstName );
				}
				AddInfo( psTable, PERSON_DISPLAY_NAME, pnRealName );
				AddInfo( psTable, PERSON_PRIVATE_PHONE, pnNumber );

				Debug( KERN_DEBUG, "AddPerson() %s\n", pnId );
				AddPerson( psTable, FALSE );
				g_free( pnId );

				g_hash_table_destroy( psTable );

				g_free( pnName );
				//g_free( pnNumber );
				//g_free( pnRealName );
				nCount++;
			}
		}
	}

	freeHandler( psHandler );

	routerLogout( psProfile );
		
	return 0;
}

/**
 * \brief Convert & to %26
 * \param pnInput input string
 * \return output string
 */
static gchar *speedfonConvert( gchar *pnInput ) {
	GRegex *psLocal = g_regex_new( "&", G_REGEX_DOTALL | G_REGEX_OPTIMIZE, 0, NULL );
	gchar *pnTmp = g_regex_replace_literal( psLocal, pnInput, -1, 0, "%26", 0, NULL );

	return pnTmp;
}

/**
 * \brief Write ebook list
 * \return error code
 */
static int speedfonWriteBook( void ) {
	struct sUrlHandler *psHandler;
	gchar anBuffer[ 1024 ];
	GList *psList = psPersonsList;
	GString *psDelete;
	GString *psEntry;
	gint nIndex;

	if ( routerLogin( getActiveProfile() ) == -1 ) {
		return -1;
	}

	/* Delete list */
	snprintf( anBuffer, sizeof( anBuffer ), "%s/cgi-bin/webcm", routerGetHost( getActiveProfile() ) );
	psHandler = urlHandler( anBuffer, 80 );
	psDelete = g_string_new( "" );

	nIndex = g_list_length( psList );
	nIndex = 100;
	Debug( KERN_DEBUG, "Length of list: %d\n", nIndex );
	while ( nIndex != 0 ) {
		g_string_append_printf( psDelete, "&telcfg:command/HotDialEntry%d=delete", --nIndex );
	}

	Debug( KERN_DEBUG, "Send str: %d [%s]\n", strlen( psDelete -> str ), psDelete -> str );
	setPostData( psHandler, psDelete -> str );

	readUrl( psHandler, getActiveProfile() );
	g_string_free( psDelete, TRUE );
	freeHandler( psHandler );

	/* Add list */
	snprintf( anBuffer, sizeof( anBuffer ), "%s/cgi-bin/webcm", routerGetHost( getActiveProfile() ) );
	psHandler = urlHandler( anBuffer, 80 );
	psDelete = g_string_new( "" );
	
	for ( nIndex = 0, psList = psPersonsList; psList != NULL && psList -> data != NULL; psList = psList -> next ) {
		struct sPerson *psPerson = psList -> data;
		gchar anCode[ 3 ];

		if ( psPerson -> nFlags & PERSON_FLAG_DELETED ) {
			psPerson -> nFlags = PERSON_FLAG_UNCHANGED;
			continue;
		}

		if ( psPerson -> pnPrivatePhone == NULL || strlen( psPerson -> pnPrivatePhone ) <= 0 ) {
			continue;
		}

		snprintf( anCode, sizeof( anCode ), "%.2d", nIndex + 1 );

		if ( psPerson -> pnId != NULL ) {
			g_free( psPerson -> pnId );
		}
		psPerson -> pnId = g_strdup_printf( "%d", nIndex );

		psEntry = g_string_new( "" );

		/* Set var:new=1 */
		g_string_append_printf( psEntry, "&var:new=1" );


		/*********/
		/** VAR **/
		/*********/

		/* Set code */
		g_string_append_printf( psEntry, "&var:code=%s", anCode );

		/* Set vanity */
		g_string_append_printf( psEntry, "&var:vanity=%s", anCode );

		/* Set phone number */
		if ( psPerson -> pnPrivatePhone != NULL && strlen( psPerson -> pnPrivatePhone ) > 0 ) {
			g_string_append_printf( psEntry, "&var:number=%s", psPerson -> pnPrivatePhone );
		}

		/* Set general name entry */
		g_string_append_printf( psEntry, "&var:name" );

		if ( speedfonGetSwap() == TRUE ) {
			/* Add last name if present */
			if ( psPerson -> pnLastName != NULL && strlen( psPerson -> pnLastName ) > 0 ) {
				gchar *pnConv = speedfonConvert( psPerson -> pnLastName );
				g_string_append_printf( psEntry, "%s", pnConv );
			}

			/* If needed, add space between first and last name */
			if ( psPerson -> pnFirstName != NULL && strlen( psPerson -> pnFirstName ) > 0 ) {
				g_string_append_printf( psEntry, " " );
			}
		}

		/* Add firstname if present */
		if ( psPerson -> pnFirstName != NULL && strlen( psPerson -> pnFirstName ) > 0 ) {
			gchar *pnConv = speedfonConvert( psPerson -> pnFirstName );
			g_string_append_printf( psEntry, "%s", pnConv );

			/* If needed, add space between first and last name */
			if ( psPerson -> pnLastName != NULL && strlen( psPerson -> pnLastName ) > 0 ) {
				g_string_append_printf( psEntry, " " );
			}
		}

		if ( speedfonGetSwap() == FALSE ) {
			/* Add last name if present */
			if ( psPerson -> pnLastName != NULL && strlen( psPerson -> pnLastName ) > 0 ) {
				gchar *pnConv = speedfonConvert( psPerson -> pnLastName );
				g_string_append_printf( psEntry, "%s", pnConv );
			}
		}

		/************/
		/** TELCFG **/
		/************/

		/* Set code */
		g_string_append_printf( psEntry, "&telcfg:settings/HotDialEntry%d/Code=%s", nIndex, anCode );

		/* Set code */
		g_string_append_printf( psEntry, "&telcfg:settings/HotDialEntry%d/Vanity=%s", nIndex, anCode );

		/* Set phone number */
		if ( psPerson -> pnPrivatePhone != NULL && strlen( psPerson -> pnPrivatePhone ) > 0 ) {
			g_string_append_printf( psEntry,
					               "&telcfg:settings/HotDialEntry%d/Number=%s", nIndex, psPerson -> pnPrivatePhone );
		}

		/* Set general name entry */
		g_string_append_printf( psEntry, "&telcfg:settings/HotDialEntry%d/Name=", nIndex );

		if ( speedfonGetSwap() == TRUE ) {
			/* Add last name if present */
			if ( psPerson -> pnLastName != NULL && strlen( psPerson -> pnLastName ) > 0 ) {
				gchar *pnConv = speedfonConvert( psPerson -> pnLastName );
				g_string_append_printf( psEntry, "%s", pnConv );
			}

			/* If needed, add space between first and last name */
			if ( psPerson -> pnFirstName != NULL && strlen( psPerson -> pnFirstName ) > 0 ) {
				g_string_append_printf( psEntry, " " );
			}
		}

		/* Add firstname if present */
		if ( psPerson -> pnFirstName != NULL && strlen( psPerson -> pnFirstName ) > 0 ) {
			gchar *pnConv = speedfonConvert( psPerson -> pnFirstName );
			g_string_append_printf( psEntry, "%s", pnConv );

			/* If needed, add space between first and last name */
			if ( psPerson -> pnLastName != NULL && strlen( psPerson -> pnLastName ) > 0 ) {
				g_string_append_printf( psEntry, " " );
			}
		}

		if ( speedfonGetSwap() == FALSE ) {
			/* Add last name if present */
			if ( psPerson -> pnLastName != NULL && strlen( psPerson -> pnLastName ) > 0 ) {
				gchar *pnConv = speedfonConvert( psPerson -> pnLastName );
				g_string_append_printf( psEntry, "%s", pnConv );
			}
		}

		if ( psDelete -> len + psEntry -> len + 22 >= nSpeedfonMaxSize ) {
			setPostData( psHandler, psDelete -> str );
			Debug( KERN_DEBUG, "write: %d/%d '%s'\n", psDelete -> len, CURL_MAX_WRITE_SIZE, psDelete -> str );
			readUrl( psHandler, getActiveProfile() );
			g_string_free( psDelete, TRUE );

			psDelete = g_string_new( "" );
		}

		g_string_append_printf( psDelete, "%s", psEntry -> str );
		g_string_free( psEntry, TRUE );

		nIndex++;
		psPerson -> nFlags = PERSON_FLAG_UNCHANGED;
	}

	if ( psDelete -> len > 100 ) {
		Debug( KERN_DEBUG, "write: %d/%d '%s'\n", psDelete -> len, CURL_MAX_WRITE_SIZE, psDelete -> str );
		//Debug( KERN_DEBUG, "write: %d/%d\n", psDelete -> len, CURL_MAX_WRITE_SIZE );
		setPostData( psHandler, psDelete -> str );

		readUrl( psHandler, getActiveProfile() );
	}
	freeHandler( psHandler );
	g_string_free( psDelete, TRUE );


	routerLogout( getActiveProfile() );

	return 0;
}

void prefsAddNone( struct sProfile *psProfile, const char *pnName );

/**
 * \brief Display speedfon preferences window
 * \param psParent parent widget
 */
static void speedfonPreferences( GtkWidget *psParent ) {
	GtkWidget *psDialog = gtk_dialog_new_with_buttons( _( "SpeedFon Preferences" ), GTK_WINDOW( psParent ), 0, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL );
	GtkWidget *psCheckBox = gtk_check_button_new_with_label( _( "Swap first/lastname" ) );
	GtkWidget *psBox;

	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( psCheckBox ), speedfonGetSwap() );

	psBox = gtk_dialog_get_content_area( GTK_DIALOG( psDialog ) );
	gtk_box_pack_start( GTK_BOX( psBox ), psCheckBox, FALSE, TRUE, 15 );

	gtk_widget_set_size_request( psDialog, 300, 80 );
	gtk_widget_show( GTK_WIDGET( psCheckBox ) );
	gtk_window_set_position( GTK_WINDOW( psDialog ), GTK_WIN_POS_CENTER_ON_PARENT );
	gtk_dialog_run( GTK_DIALOG( psDialog ) );

	prefsAddNone( getActiveProfile(), "/plugins/speedfon" );

	speedfonSetSwap( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( psCheckBox ) ) );
	gtk_widget_destroy( psDialog );
	SavePreferences( getActiveProfile() );

	freePersons();
	speedfonReadBook();
}

/** book definition */
static struct sAddressBook sSpeedFon = {
	speedfonReadBook,
	speedfonWriteBook,
	PERSON_FIRST_NAME | PERSON_LAST_NAME | PERSON_DISPLAY_NAME |
	PERSON_PRIVATE_PHONE,
	PERSON_FIRST_NAME | PERSON_LAST_NAME | PERSON_DISPLAY_NAME |
	PERSON_PRIVATE_PHONE,
	speedfonPreferences
};

MODULE_INIT( PLUGIN_TYPE_BOOK, _( "SpeedFon Addressbook" ), &sSpeedFon, NULL, NULL );
